<?php
if (!defined('ABSPATH')) {
    exit;
}

function api_post_grid_get_posts(WP_REST_Request $request)
{
    
    $post_type = sanitize_text_field($request->get_param('post_type')) ?: 'post';
    $taxonomy = sanitize_text_field($request->get_param('taxonomy'));
    $term_id = absint($request->get_param('term_id'));
    $per_page = absint($request->get_param('per_page')) ?: 8;
    $page = absint($request->get_param('page')) ?: 1;
    $order = in_array($request->get_param('order'), ['ASC', 'DESC']) ? $request->get_param('order') : 'DESC';
    $orderby = sanitize_text_field($request->get_param('orderby')) ?: 'date';

    error_log('Received parameters: ' . print_r([
        'post_type' => $post_type,
        'taxonomy' => $taxonomy,
        'term_id' => $term_id,
        'per_page' => $per_page,
        'page' => $page,
        'order' => $order,
        'orderby' => $orderby
    ], true));

    $args = [
        'post_type' => $post_type,
        'posts_per_page' => $per_page,
        'paged' => $page,
        'order' => $order,
        'orderby' => $orderby,
        'post_status' => 'publish'
    ];

    if (!empty($taxonomy) && !empty($term_id)) {
        $args['tax_query'] = [
            [
                'taxonomy' => $taxonomy,
                'field' => 'term_id',
                'terms' => $term_id,
                'include_children' => true
            ]
        ];
    }

    error_log('WP_Query arguments: ' . print_r($args, true));

    $query = new WP_Query($args);
    $posts = [];

    if ($query->have_posts()) {
        while ($query->have_posts()) {
            $query->the_post();
            $posts[] = [
                'id' => get_the_ID(),
                'title' => get_the_title(),
                'link' => get_permalink(),
                'image' => get_the_post_thumbnail_url(get_the_ID(), 'full'),
                'image_thumbnail' => get_the_post_thumbnail_url(get_the_ID(), 'medium'),
                'excerpt' => get_the_excerpt()
            ];
        }
    }

    wp_reset_postdata();

    error_log('Found Posts: ' . count($posts));

    return rest_ensure_response([
        'success' => true,
        'data' => [
            'posts' => $posts,
            'total' => (int) $query->found_posts,
            'max_pages' => (int) $query->max_num_pages,
            'current_page' => (int) $page,
            'per_page' => (int) $per_page
        ]
    ]);
}

function api_post_grid_get_post_types()
{
    $post_types = get_post_types(['public' => true], 'objects');
    $data = [];

    foreach ($post_types as $post_type) {
        
        if (!in_array($post_type->name, ['attachment', 'revision', 'nav_menu_item'])) {
            $data[] = [
                'value' => $post_type->name,
                'label' => $post_type->label
            ];
        }
    }

    return rest_ensure_response([
        'success' => true,
        'data' => $data
    ]);
}

function api_post_grid_get_taxonomies($request)
{
    $post_type = sanitize_text_field($request['post_type']);

    if (!post_type_exists($post_type)) {
        return new WP_Error(
            'invalid_post_type',
            'Invalid post type specified',
            array('status' => 404)
        );
    }

    $taxonomies = get_object_taxonomies($post_type, 'objects');
    $data = [];

    foreach ($taxonomies as $taxonomy) {
        
        if ($taxonomy->public && $taxonomy->publicly_queryable) {
            $data[] = [
                'value' => $taxonomy->name,
                'label' => $taxonomy->labels->singular_name 
            ];
        }
    }

    return rest_ensure_response([
        'success' => true,
        'data' => $data
    ]);
}

function api_post_grid_get_terms($request)
{
    $taxonomy = sanitize_text_field($request['taxonomy']);

    if (!taxonomy_exists($taxonomy)) {
        return new WP_Error(
            'invalid_taxonomy',
            'Invalid taxonomy specified',
            array('status' => 404)
        );
    }

    $args = [
        'taxonomy' => $taxonomy,
        'hide_empty' => false,
        'orderby' => 'name',
        'order' => 'ASC'
    ];

    $terms = get_terms($args);

    if (is_wp_error($terms)) {
        return rest_ensure_response([
            'success' => false,
            'message' => $terms->get_error_message()
        ]);
    }

    $data = [];
    foreach ($terms as $term) {
        $data[] = [
            'value' => $term->term_id, 
            'label' => $term->name,
            'count' => $term->count 
        ];
    }

    return rest_ensure_response([
        'success' => true,
        'data' => $data
    ]);
}